function [is ix iy iz] = calc_all(H,zh,dx,f)
%
% Beregner f?rst intensitet ved kilden, is, s? trykk og de tre
% partikkelhastighetskomponentene i rommet.
%
% H = Komplekst lydtrykk
% zh = Hoyde holografiplan
% dx = Mikrofonavstand i x- og y-retning
% f = Frekvens
%

c = 340; 
p0 = 1.21;
Fs = 48000;
Lx = 13*dx; % (N+1)*dx
Ly = 13*dx; % (N+1)*dy
Nx = 24; % Antall punkter i x-retning. (2x antall maalepunkter)
Ny = 24; % Antall punkter i y-retning. (2x antall maalepunkter)
Nz = 200; % Antall punkter som skal beregnes i z-retning
dz = dx/10; % Avstand mellom hvert z-plan

X1 = Nx/4 + 1; % Startpunkt for "virkelige" mikrofonposisjoner
X2 = 3*Nx/4; % Sluttpunkt for "virkelige" mikrofonposisjoner

% Oppretter ymse beregningsvariabler
dim = [Nx Nx Nz];
ps = zeros(Nx,Ny);
vs = zeros(Nx,Ny);
Ps = zeros(Nx,Ny);
Vs = zeros(Nx,Ny);
Ds = zeros(Nx,Ny);
H_fft2 = zeros(Nx,Ny);
H_p = zeros(Nx,Ny);

% Finner riktig index for den gitte frekvensen
n = f/((Fs/2)/(length(H)/2));
H_p(X1:X2,X1:X2) = H(:,:,n);

% Kantfilter
W =  tukeywin(Nx/2,.3)*tukeywin(Ny/2,.3)';
H_p(X1:X2,X1:X2) = (H_p(X1:X2,X1:X2)).*W(:,:);
H_fft2(:,:) = (fft2(H_p(:,:)));

    for x=-Ny/2:Nx/2-1
    for y=-Ny/2:Ny/2-1
       kx = 2*pi*x/Lx; 
       ky = 2*pi*y/Ly;
       k = 2*pi*f/c;
       
       % Setter verdier for k-romsfilteret
       kr = sqrt(kx^2 + ky^2);
       kc = 0.6*12*pi/Lx;
       a = 0.2;
       
       % Finner verdi for kz
        if k^2 >= (kx^2 + ky^2)
            kz = sqrt(k^2 - kx^2 - ky^2);
        else
            kz = 1i*sqrt(ky^2 + kx^2 - k^2);
        end
        
       % k-romsfilter
        if abs(kr) <= kc
            Wk = 1 - 0.5*exp((kr/kc - 1)/a);
        elseif abs(kr) > kc
            Wk = 0.5*exp((1 - kr/kc)/a);
        else
            Wk = 1;
        end
        
        % Zero-padding av Greensfunksjon
        if (x>=-6 && x<=5) && (y>=-6 && y<=5)
            Gp = exp(1i*kz*zh);
            Gv = (1/(p0*c*k))*exp(1i*kz*zh);
        else
            Gp = 0;
            Gv = 0;
        end
       
       Ps(x+1+(Nx/2),y+1+(Ny/2)) = H_fft2(x+1+(Nx/2),y+1+(Ny/2)).*Gp.*Wk;
       Vs(x+1+(Nx/2),y+1+(Ny/2)) = H_fft2(x+1+(Nx/2),y+1+(Ny/2)).*Gv.*Wk;
       
       %[thet,phi,r] = cart2sph(x,y,zh);
       %Ds(x+1+(Nx/2),y+1+(Ny/2)) = k*cos(thet)*H_fft2(x+1+(Nx/2),y+1+(Ny/2))*1i*exp(-1i*k*cos(zh));
       
    end
    end
    ps(:,:) = ifft2((Ps(:,:)),Nx,Ny); % Trykk ved kilden
    vs(:,:) = ifft2((Vs(:,:)),Nx,Ny); % Partikkelhastighet ved kilden i z-retning
    %ds(:,:) = ifft2((Ds(:,:)),Nx,Ny);

is = real(ps.*conj(vs))/2; % Intensitet ved kilden i z-retning

% Oppretter ymse beregningsvariabler
Pz = zeros(dim);
Vz = zeros(dim);
Vx = zeros(dim);
Vy = zeros(dim);
pz = zeros(dim); % Trykk i rommet
vz = zeros(dim); % Partikkelhastighet z-retning
vx = zeros(dim); % Partikkelhastighet x-retning
vy = zeros(dim); % Partikkelhastighet y-retning

% Regner ut verdier for hvert enkelt z-plan
for z=1:Nz
    zi = z*dz; % Fysisk h?yde over planet
    for x=-Nx/2:Nx/2-1
    for y=-Ny/2:Ny/2-1
       kx = 2*pi*x/Lx;
       ky = 2*pi*y/Ly;
       k = 2*pi*f/c;
       
       % Setter verdier for k-romsfilteret
       kr = sqrt(kx^2 + ky^2);
       kc = 0.6*12*pi/Lx;
       a = 0.2;
       
        % Finner verdi for kz
        if k^2 >= (kx^2 + ky^2)
             kz = sqrt(k^2 - kx^2 - ky^2);
        else
             kz = 1i*sqrt(ky^2 + kx^2 - k^2);
        end
        
        % k-romsfilter
        if abs(kr) <= kc
             Wk = 1 - 0.5*exp((kr/kc - 1)/a);
        elseif abs(kr) > kc
             Wk = 0.5*exp((1 - kr/kc)/a);
        else
             Wk = 1;
        end
        
        % Zero-padding av Greensfunksjon
        if (x>=-6 && x<=5) && (y>=-6 && y<=5)
            Gp = exp(1i*kz*(zi));
            Gv = (1/(p0*c*k))*exp(1i*kz*(zi));
        else
            Gp = 0;
            Gv = 0;
        end
        
       Pz(x+1+(Nx/2),y+1+(Ny/2),z) = Ps(x+1+(Nx/2),y+1+(Ny/2)).*Gp;%.*Wk;
       Vz(x+1+(Nx/2),y+1+(Ny/2),z) = Ps(x+1+(Nx/2),y+1+(Ny/2)).*kz.*Gv;%.*Wk;
       Vx(x+1+(Nx/2),y+1+(Ny/2),z) = Ps(x+1+(Nx/2),y+1+(Ny/2)).*kx.*Gv;%.*Wk;
       Vy(x+1+(Nx/2),y+1+(Ny/2),z) = Ps(x+1+(Nx/2),y+1+(Ny/2)).*ky.*Gv;%.*Wk;
       
    end
    end
    pz(:,:,z) = ifft2((Pz(:,:,z)),Nx,Ny);
    vz(:,:,z) = ifft2((Vz(:,:,z)),Nx,Ny);
    vx(:,:,z) = ifft2((Vx(:,:,z)),Nx,Ny);
    vy(:,:,z) = ifft2((Vy(:,:,z)),Nx,Ny);
end
ix = real(pz.*conj(vx))/2;
iy = real(pz.*conj(vy))/2;
iz = real(pz.*conj(vz))/2;